﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

/************************************************************
 * FileStuff.cs  July 2011									*
 * Civ4 HUD Color Selector Version 2                        *
 *															*
 * - A file utility program for altering the Civ 4 HUD		*
 * color using sliders and a visual display of the color	*
 *															*
 ***********************************************************/


namespace FileStuff
{
	public partial class FileStuff : Form
	{
		// Some globals to avoid parameter swapping
		public string regPath, filePath;
		public string[ ] fileContents;
		bool makeBak = false;
		
		
		public struct TempColor
		{
			// A structure to hold the temporary color values.
			public string orgRed;
			public string orgGreen;
			public string orgBlue;
			public string orgAlpha;
			public int iRed;
			public int iGreen;
			public int iBlue;
			public int iAlpha;

		}

		// Instantiate the TempColor Struct globally
		TempColor tColor;
		
		
		public FileStuff()
		{
			InitializeComponent();
			

			// Force the window controls double buffering value to be optimized
			// using ControlStyles bitflags
			this.SetStyle( ControlStyles.OptimizedDoubleBuffer |
			   ControlStyles.UserPaint |
			   ControlStyles.AllPaintingInWmPaint,
			   true );
			this.UpdateStyles();

			
			// Get the file paths, foldeers and whatnot to locate the theme file that we need 			
			fileOps workingFile = new fileOps ();
			
			// Get the game's install path from the registry
			regPath = workingFile.getFilePathReg();

			// If the program can't find Civ4...
			if ( String.IsNullOrEmpty( regPath ) )
			{
				// Display the error msg
				displayStatus( "Can't find any version of Civ 4." );

				// Display the msg box
				string query = "The program was unable to locate the \n" +
						"Civ 4 install directory, and cannot continue. \n" +
						"Select OK to quit the program.";
				if ( MessageBox.Show( query, "Can't Find Civ 4", MessageBoxButtons.OK ) == DialogResult.OK )
				{
					// No game, so app aborts
					Close();
				}
			}

			// If the registry path is not null, assemble the complete file path
			filePath = workingFile.locateFullPath( regPath );

			// Check to make sure that a bak file exists, otherwise make one
			makeBak = doesBakFileExist( workingFile, filePath );

			// Now that the bak file exists, read the contents from the theme file
			if ( makeBak )
			{
				fileContents = workingFile.readInFile( filePath );
				
			}

			// Get the HUD color information from the file and display it.
			bool results = getResults( fileContents );

			// If the information that we need is read successfully, display the results
			if ( results )
			{
				displayOriginalResults();
			}
			
			// Otherwise, the read failed, show message box
			else
			{
				displayStatus( "Load failure." );
				string query = "The program was unable to load any values from the \n" +
						"working file.  You may continue making new values by selecting Yes \n" +
						" or select No to quit the program.  Do you wish to continue?";
				if ( MessageBox.Show( query, "Load Error", MessageBoxButtons.YesNo ) == DialogResult.No )
				{
					Close();
				}

			}
			
			
		}


		#region - file based methods


		// Check to see if the file has been backed up to a .BAK file
		// If not, make the backup so that we only mess up the working copy 
		private bool doesBakFileExist( fileOps workingFile, string pathname )
		{
			
			string filePath = String.Empty, tempThmFile = pathname;

			// Send user message
			displayStatus( "Looking for \".bak\" file..." );
			
			// Set the pathname to have a .bak extension and then check the folder for the backup version
			pathname = Path.ChangeExtension( pathname, ".bak" );
			filePath = workingFile.locateFile( pathname );

			if ( !File.Exists( filePath ) )
			{
				// If the .bak file doesn't exist, then make a copy of the .thm
				displayStatus( "Backup file does not exist\n\r  Attempting to make copy now" );

				// Try - Catch block for Exception handling on file error
				// this should be fairly self-explanitory
				try
				{
					string query = "You are about to make a backup copy of " + tempThmFile + "\n" +
						"Do you wish to continue?";
					if ( MessageBox.Show( query, "Make Backup", MessageBoxButtons.YesNo ) == DialogResult.Yes )
					{
						File.Copy( tempThmFile, pathname );
						MessageBox.Show( "The backup has been made.", "File Copied", MessageBoxButtons.OK );
						displayStatus( "Backup sucessful." );
						
						return true;
					}
				}
				// If it blows up in our face...
				catch ( Exception ex )
				{
					MessageBox.Show( "Unable to make a backup copy of the file.  The following exception has" +
						" occurred: " + ex.Message, "File Copy Failed", MessageBoxButtons.OK );
					displayStatus( "Backup failed!" );

				}

				return false;  //  false - no file found
			}

			else
			{
				// if the .bak file is actually exists, set message and return true
				displayStatus( "Backup was previously made in another session.\n\r  You are using the working copy." );
				return true;
				
				
			}
					
		}


		// Reads the color data from the theme file contained in the fileContents string array
		private bool getResults( string[ ] fileContents )
		{
			bool success = false;

			// Starts at line 199 (Array element 200) in case of some unexpected in-file changes
			short linePointer = 200;

			// Create a string to receive the line when it is found and parsed
			string lineContent = " ";  

			// Create a string to match in line search - this is the actual text in the file from the
			// beginning of the line we need to parse
			string colorString = "\t\t\t\tGColor\t.BG\t\t\t\t\t\t\t=\tGColor(";
			
			// Search the filecontents array
			// Loop through the elements and compare them with the colorString value
			// Initially, the lineContent string is just a space character
			while ( !lineContent.StartsWith( colorString ) )   
			{
				// Get the correct line in the file for the changes by iterating with linePointer
				lineContent = fileContents[linePointer];  
				linePointer++;
			}

			// When the string is found...
			if ( lineContent.StartsWith( colorString ) )
			{
				
				// Set the lineIndex value to the occurence of the open bracket in the string
				int lineIndex = lineContent.IndexOf( '(' );

				// Move along the string, picking up the four three-character values, skipping the commas
				// Handle the red channel data
				tColor.orgRed = lineContent.Substring( lineIndex + 1, 3 );
				tColor.iRed = Convert.ToInt16( tColor.orgRed );

				// Handle the green channel data
				tColor.orgGreen = lineContent.Substring( lineIndex + 5, 3 );
				tColor.iGreen = Convert.ToInt16( tColor.orgGreen );

				// Handle the blue channel data
				tColor.orgBlue = lineContent.Substring( lineIndex + 9, 3 );
				tColor.iBlue = Convert.ToInt16( tColor.orgBlue );

				// Handle the alpha channel data
				tColor.orgAlpha = lineContent.Substring( lineIndex + 13, 3 );
				tColor.iAlpha = Convert.ToInt16( tColor.orgAlpha );

				// Set the success tag and the display status string
				success = true;
				displayStatus( "File contents successfully retrieved" );

			}

			return success;
		}

		
		// Writes the new values into the fileContents array, then writes that
		// information to the working file
		private bool setResults()
		{
			bool success = false;

			// Parses the correct line and loads an array for the boxes
			short linePointer = 200;
			string lineContent = " ";  // Create a string to receive the line when it is found
			string colorString = "\t\t\t\tGColor\t.BG\t\t\t\t\t\t\t=\tGColor(";
			
			// Make a small string array to help with space padding, and load the textBox values into it
			string[ ] boxFiller = new string [4];
			boxFiller[0] = textBox14.Text;  // Red
			boxFiller[1] = textBox15.Text;  // Green
			boxFiller[2] = textBox16.Text;  // Blue
			boxFiller[3] = textBox17.Text;  // Alpha

			// Search the filecontents array to get the proper line to change
			// Initially, the lineContent string is empty
			while ( !lineContent.StartsWith ( colorString ) )   
			{
				lineContent = fileContents[linePointer];  // Get the correct line in the file for the changes
				linePointer++;
			}

			if ( lineContent.StartsWith( colorString ) )
			{
				// Load the index position of the first "(" character in the line
				int lineIndex = lineContent.IndexOf( '(' );

				// Create some temporary strings to hold file data to be replaced
				string tempRed, tempGreen, tempBlue, tempAlpha;

				// Get the old file colors from the file for replacement values
				tempRed = lineContent.Substring( lineIndex + 1, 3 );
				tempGreen = lineContent.Substring( lineIndex + 5, 3 );
				tempBlue = lineContent.Substring( lineIndex + 9, 3 );
				tempAlpha = lineContent.Substring( lineIndex + 13, 3 );
				int x = 0;
				
				// Pad out the strings in the data to be written so that Civ can still read the file correctly
				// This is IMPORTANT
				foreach ( string s in boxFiller )
				{
					if ( s.Length < 2 )
					{
						boxFiller[x] = s.Insert( 0, "  " );
					}
					else if ( s.Length == 2 )
					{
						boxFiller[x] = s.Insert( 0, " " );
					}
					x++;
				}
				
				// Now replace those temp strings with the new values
				lineContent = lineContent.Replace( tempRed, boxFiller[0] );
				lineContent = lineContent.Replace( tempGreen, boxFiller[1] );
				lineContent = lineContent.Replace( tempBlue, boxFiller[2] );
				lineContent = lineContent.Replace( tempAlpha, boxFiller[3] );

				// Back up one indice and replace the color string with the new contents of the line
				fileContents[linePointer - 1] = lineContent; 

				// Do the file write
				fileOps workingFile = new fileOps();

				// Call the Write method
				if ( workingFile.writeOutFile( filePath, fileContents ) )
				{
					success = true;
					displayStatus( "Values successfully written to working file." );
				}

				// No error handling here because of the exception method in the actual file writing method
			}

			return success;
		}

		
	#endregion

		


		

		#region - display related methods

		// These are methods related to the display of the data and the color swatches

		
		// This method is called every time the color changes
		private void updateSwatch( int alpha, int red, int green, int blue )
		{
			// Paint the HUD sample swatch
			// Note that the rectangle paint does not happen until the sliders are moved - hmmmmm...
			using ( Graphics g = this.CreateGraphics() )
			{
				Rectangle rect = new Rectangle( 824, 275, 123, 100 );
				SolidBrush brush = new SolidBrush( Color.FromArgb( alpha, red, green, blue ) );
				
				g.FillRectangle( brush, rect );
				g.Flush();
			}

			// Set the back color of the overlay panel
			this.panel2.BackColor = Color.FromArgb( alpha, red, green, blue );

		}


		// Display the original results that were loaded from the file
		// Called from the resetButtonClick event and on program start
		private void displayOriginalResults()
		{
			// Refresh the "Original" textboxes
			textBox10.Text = tColor.orgRed;
			textBox11.Text = tColor.orgGreen;
			textBox12.Text = tColor.orgBlue;
			textBox13.Text = tColor.orgAlpha;

			// udate the  new color textboxes with the original values
			textBox14.Text = tColor.orgRed;
			textBox15.Text = tColor.orgGreen;
			textBox16.Text = tColor.orgBlue;
			textBox17.Text = tColor.orgAlpha;

			// Set the labels on the trackbars
			redTrackVal.Text = tColor.orgRed;
			greenTrackVal.Text = tColor.orgGreen;
			blueTrackVal.Text = tColor.orgBlue;
			alphaTrackVal.Text = tColor.orgAlpha;

			// The next four lines set the tColor integer fields and the trackbar
			// values to the same value at the same time
			// Puts everything back to "zero" or the start condition
			tColor.iRed = redTrackBar.Value = Convert.ToInt16( tColor.orgRed );
			tColor.iGreen = greenTrackBar.Value = Convert.ToInt16( tColor.orgGreen );
			tColor.iBlue = blueTrackBar.Value = Convert.ToInt16( tColor.orgBlue );
			tColor.iAlpha = alphaTrackBar.Value = Convert.ToInt16( tColor.orgAlpha );

			// Reset the swatch to the original color
			updateSwatch( tColor.iAlpha, tColor.iRed, tColor.iGreen, tColor.iBlue );
			
		}



		// Writes the operating status messages to the large text box followed by CR/LF/LF
		private void displayStatus( string opStatus )
		{
			textBox5.AppendText ( "--> " + opStatus );
			textBox5.AppendText( "\r\n\n" );
		}

		#endregion

		#region - Button and control methods
		
		
		// This method captures the color that is clicked in the rainbow color chooser
		private void pictureBox1_MouseClick( object sender, MouseEventArgs e )
		{

			// Equate the bitmap with the one in the picturebox and get the color under the mouse cursor
			Bitmap MyBitmap = new Bitmap (pictureBox1.Image );
			Color PixelColor = MyBitmap.GetPixel( e.X, e.Y );

			// Set the colors in the Color structure
			tColor.iAlpha = PixelColor.A;
			tColor.iRed = PixelColor.R;
			tColor.iGreen = PixelColor.G;
			tColor.iBlue = PixelColor.B;
			// Update the color swatch
			updateSwatch( tColor.iAlpha, tColor.iRed, tColor.iGreen, tColor.iBlue );

			// Release the memory used by the temporary bitmap
			MyBitmap.Dispose();
			
		}

		// Resets the sliders and swatch to beginning results
		private void resetButton_Click( object sender, EventArgs e )
		{
			// Get the results from the file again
			bool results = getResults( fileContents );

			if ( results )
			{
				displayOriginalResults();
			}
		}

		
		// This restores the original file from the bak file that was made
		private void undoAllButton_Click( object sender, EventArgs e )
		{
			// Create a working file object
			fileOps workingFile = new fileOps();

			// Gets the data and writes a message if successful
			// It also changes the swatch colors back to original
			if ( workingFile.restoreFile( filePath ) )
			{
				fileContents = workingFile.readInFile( filePath );
				bool results = getResults( fileContents );

				if ( results )
				{
					displayOriginalResults();
					displayStatus( "Original theme file restored" );
				}
				
			}
			
			// If it fails, nothing happens
			else
			{
				displayStatus( "Original theme file restoration failed." );
			}
		}

		
		// Save the new color that has been generated to the theme file
		private void saveButton_Click( object sender, EventArgs e )
		{
			// Make sure that the user really wants to save this color
			string query = "You are about to write the new HUD color values.\n" +
						"Do you wish to continue?";
			if ( MessageBox.Show( query, "Write New Values", MessageBoxButtons.YesNo ) == DialogResult.Yes )
			{
				// If the user answers yes, then save the color, otherwise bail out
				setResults();
				MessageBox.Show( "The new HUD color values have been written to file.", "File Updated", MessageBoxButtons.OK );
				displayStatus( "Write of new values sucessful.\n" );

			}

			else
			{
				displayStatus( "File save cancelled." );
			}
			
		}

		// Pretty obvious
		private void exitButton_Click( object sender, EventArgs e )
		{
			Close();
		}

		
		// The trackbar scroll event handlers all update their particular values in the color structure
		// And then call updateSwatch to change the display
		
		private void redTrackBar_Scroll( object sender, EventArgs e )
		{
			tColor.iRed = redTrackBar.Value;
			redTrackVal.Text = redTrackBar.Value.ToString();
			textBox14.Text = tColor.iRed.ToString();

			updateSwatch( tColor.iAlpha, tColor.iRed, tColor.iGreen, tColor.iBlue );

		}


		private void greenTrackBar_Scroll( object sender, EventArgs e )
		{
			tColor.iGreen = greenTrackBar.Value;
			greenTrackVal.Text = greenTrackBar.Value.ToString();
			textBox15.Text = tColor.iGreen.ToString();

			updateSwatch( tColor.iAlpha, tColor.iRed, tColor.iGreen, tColor.iBlue );
		}

		private void blueTrackBar_Scroll( object sender, EventArgs e )
		{
			tColor.iBlue = blueTrackBar.Value;
			blueTrackVal.Text = blueTrackBar.Value.ToString();
			textBox16.Text = tColor.iBlue.ToString();

			updateSwatch( tColor.iAlpha, tColor.iRed, tColor.iGreen, tColor.iBlue );
		}

		private void alphaTrackBar_Scroll( object sender, EventArgs e )
		{
			tColor.iAlpha = alphaTrackBar.Value;
			alphaTrackVal.Text = alphaTrackBar.Value.ToString();
			textBox17.Text = tColor.iAlpha.ToString();
			

			updateSwatch( tColor.iAlpha, tColor.iRed, tColor.iGreen, tColor.iBlue );
		}

		#endregion

		
		
		// End program code
		
	}



}
